##==============================================================================
# CS150: Spring 2010
# MIPS150 Checkpoint 1 (part 2) Checkoff Test suite
#
# Part2: Covers rest of instructions: SLTI, SLTIU, XORI, SRL, SRA, SRLV, AND, OR, 
#        XOR, NOR, SLTU, J, JALR, BLEZ, BGTZ, BLTZ, BGEZ.
#
#	See part1 for more detailed instructions on using these tests and writing
#   a simulation testbench.  This test suite is intended to do basic tests of
#   instructions not covered in part1.  You may want to pass certain tests in
#   part1 before expecting to all of part2, as certain instructions like ORI,
#   ADDIU, SUBU, BEQ, and BNE are used here.
#   
#   If all tests pass then $26 will contain BA5EBA11.  Look at the last section
#   of the code to see what other result registers should contain.  A mistake
#   in one of these registers should point you to the test that failed.
#   Comments throughout the code are meant to help you find where tests
#   may have gone wrong.
#
#   Again, this is not a comprehensive test, although it does test basic 
#   functionality of all the mentioned instructions.  Your CPU will be tested
#   with real code at later checkpoints and final checkoff.
##==============================================================================

##==============================================================================
## Revision History
##==============================================================================
## Revision B - Brandon Myers (2010/03/28)
##  Wrote description and cleaned up first two tests to eliminate cross-test
##  dependency on $18.
##
## Revision A - Brandon Myers (2010/03/28) 
##	Wrote testbench.  Tests 16-25 and final verification.
##==============================================================================

##==============================================================================
# Test setup
##==============================================================================
#
# Clear saved registers
#
ori		$16, $0, 0x0
ori		$17, $0, 0x0
ori		$18, $0, 0x0
ori		$19, $0, 0x0
ori		$20, $0, 0x0
ori		$21, $0, 0x0
ori		$22, $0, 0x0
ori		$23, $0, 0x0
ori		$24, $0, 0x0
ori		$25, $0, 0x0

##==============================================================================
# Test 16, 17
##==============================================================================
#
# Test basic instructions, with some bypassing
#
xor    $8, $0, $0
and    $9, $0, $0
ori    $8, $8, 0x01B4
ori    $9, $9, 0xAAAB
xor    $16, $8, $9       # $16=ABlF 

and    $8,  $8, $16      # $8=114
lui    $10, 0x5CA1
or     $17, $10, $16     # $17=5CA1AB1F
subu   $17, $17, 1       # $17=5CA1AB1E
xori   $10, $8, 0x0FFF   # $10 = EEB
xori   $9, $9, 0x5555    # $9 = FFFF
nor    $9, $0, $9        # $9 = FFFF0000
nor    $9, $17, $9       # $9 = 54E1
or    $9, $8, $9         # $9 = 55F5
xori  $8, $9, 0xFEEB     # $8 = AB1E
ori   $10, $0, 0x1       # $10 = 1
xor   $10, $8, $16       # $10 = 0
bne   $10, $0, FAIL16    #
nop
subu  $16, $16, 1		 # $16 = AB1E

FAIL16:

##==============================================================================
# Test 18, 19
##==============================================================================
#
# Test branches
#
blez  $18, BLEZ1TAKE   # blez and bgez tests $18=0
nop
ori   $8, $0, 0xF      # should not execute
subu  $18, $18, $8     # should not execute

BLEZ1TAKE:
bgez $18, BGEZ1TAKE
nop
ori  $8, $0, 0xFF      # should not execute
subu $18, $18, $8      # should not execute

BGEZ1TAKE:
ori  $8, $0, 0xAA
bltz $0, BLTZ1FAIL
addiu $8, $8, 0x1      # $8 = AB
bltz $8, BLTZ2FAIL
addu $18, $8, $18      # $18 = AB
blez $18, BLEZ2FAIL
nop
bgtz $0, BGTZ1FAIL
ori  $8, 0xF
sll  $8, $8, 28        # $8 = F0000000
bgtz $8, BGTZ2FAIL
sll  $18, $18, 19      # $18 = 05580000
bgtz $18, BGTZ3TAKE
sll  $18, $18, 1       # $18 = 0AB00000
ori  $18, $0,  0x10      # should not execute ($18=0x10)

BLTZ1FAIL:
BLTZ2FAIL:
BLEZ2FAIL:
BGTZ1FAIL:
BGTZ2FAIL:    
ori  $18, $0, 0x2       # should not execute ($18=2)

BGTZ3TAKE:
sll  $18, $18, 4       # $18 = AB000000
ori  $19, 0xAB1E
sll  $19, $19, 16      # $19 = AB1E0000
bltz $19, BLTZ3TAKE
nop
ori  $19, $0, 0x4       # should not execute ($19=4)

BLTZ3TAKE:
blez $19, BLEZ2TAKE
nop
ori  $19, $0, 0x8       # should not execute ($19=8)

BLEZ2TAKE:


##==============================================================================
# Test 20
##==============================================================================
#
# Test SLT instructions
#
ori		$8, $0, 0x0
slti	$8, $19, -100   # $8 = 1
ori     $9, $0, 0x0
sltiu   $9, $19, -100   # $9 = 1
ori     $10, $0, 0x1
sltiu   $10, $19, 100    # $10 = 0
ori     $12, $0, 0xF
sll     $12, $12, 28
ori     $13, $0, 0xF
sll     $13, $13, 26
ori     $11, $0, 0x1
slt     $11, $13, $12    # $11 = 0
ori     $14, $0, 0x0
sltu    $14, $13, $12    # $14 = 1

ori     $12, $0, 0x1234
ori     $15, $0, 0x1
slti    $15, $12, -1     # $15 = 0
ori     $13, $0, 0x0
sltiu   $13, $12, -1     # $13 = 1

or      $20, $20, $8     # $20 = 0x1
sll     $20, $20, 1
or      $20, $20, $9     # $20 = 0x3
sll     $20, $20, 1
or      $20, $20, $10    # $20 = 0x6
sll     $20, $20, 1      # $20 = 0xC
or      $20, $20, $11    # $20 = 0xC
sll     $20, $20, 1      # $20 = 0x18
or      $20, $20, $14    # $20 = 0x19
sll     $20, $20, 1      # $20 = 0x32
or      $20, $20, $15    # $20 = 0x32
sll     $20, $20, 1      # $20 = 0x64
or      $20, $20, $13    # $20 = 0x65

##==============================================================================
# Test 21
##==============================================================================
#
# Test SRL
#
lui     $21, 0xB000
srl     $21, $21, 1   # $21 = 0x58000000
srl     $21, $21, 2
srl     $21, $21, 3
srl     $21, $21, 4
srl     $21, $21, 5
srl     $21, $21, 6
srl     $21, $21, 7
sll     $21, $21, 8   # notice SLL; $21 = 0x0B00
ori     $21, $21, 0xA01E

##==============================================================================
# Test 22
##==============================================================================
#
# Test SRLV (with one SRL instruction also)
#
lui     $22, 0xA100
or      $8, $0, $0
srlv    $22, $22, $8
addiu   $8, $8, 3
srlv    $22, $22, $8
addiu   $8, $8, 9
srlv    $22, $22, $8
andi    $8,  $8, 0x8
srlv    $22, $22, $8
srl     $8, $8, 3
srlv    $22, $22, $8
lui     $9, 0x5C00
srl     $9, $9, 20
sll     $9, $9, 15
lui     $8, 0xB101
srl     $8, $8, 28
srlv    $9, $9, $8
or      $22, $9, $22
sll     $22, $22, 16


##==============================================================================
# Test 23, 24
##==============================================================================
#
# Test J (jump)
#
j 		JUMP1
ori		$23, $0, 0xAB00     # delay slot
ori     $23, $23, 0xFFFF    # don't execute ($23=FFFF)

JUMP1:
j		JUMP2
nop
ori		$23, $0, 0xDDDD     # don't execute ($23=DDDD)
nop
nop
nop
nop
nop
nop
nop
nop
nop
nop  # fillers for address distancing
nop
nop
nop
nop
nop
nop
nop
nop
nop
nop

JUMP2:
ori		$23, $23, 0x1E     # $23 = AB1E
j		FOO
nop
ori		$23, $0, 0x10		# don't exec ($23 = 0x10)

FOO:
addiu	$24, $0, 0x1000
addiu   $24, $24, 0x2000
addiu   $24, $24, 0x2C00
j		FOO2
nop
ori		$24, $0, 0x10		# don't exec ($24 = 0x10)

FOO2:
ori		$24, $24, 0x0020
nop
ori		$24, $24, 0x0080
nop
ori		$24, $24, 0x0000
nop
ori		$24, $24, 0x0001
sll		$24, $24, 16		# $24 = 0x5CA10000

##==============================================================================
# Test 25
##==============================================================================
#
# Test JALR
#
la		$8, BAR
JALR1:
jalr	$9, $8
ori		$25, $0, 0x0001
sll		$25, $25, 16	# don't exec ($25 = 0x00010000)

BAR:
la 		$10, JALR1
addiu	$10, $10, 8
bne		$9, $10, JALRFAIL
nop
ori 	$25, $25, 0x5ca0
sll		$25, $25, 16		# $25 = 0x5CA10000
ori		$25, $25, 0xA000	# $25 = 0x5CA1A000

la		$9, JALR2
addiu	$9, $9, 8
la		$10, BAZ
beq		$0, $0, JALR2
nop

BAZ:
addu	$8, $0, $0
bne		$8, $0, JALRZEROFAIL
ori		$25, $25, 0x18		# $25 = 0x5CA1AB18
ori		$25, $25, 0x04		# $25 = 0x5CA1AB1C
ori		$25, $25, 0x02		# $25 = 0x5CA1AB1E
nop
nop
nop
nop
nop
beq		$0, $0, JDONE
nop

JALR2:
jalr	$0, $10             # test $0 immune, and backwards jalr
ori		$25, $25, 0x0B00    # $25 = 0x5CA1AB00
ori     $25, $0, 0xBBBB     # don't exec ($25 = 0xBBBB)

JALRZEROFAIL:
JALRFAIL:
ori		$25, $0, 0x1111		# don't exec ($25 = 0x1111 or other if JALRZEROFAIL) 

JDONE:



##==============================================================================
# Test Teardown: Verify that all of the tests passed
##==============================================================================

# $16 = 0x0000AB1E
# $17 = 0x5CA1AB1E
# $18 = 0xAB000000
# $19 = 0xAB1E0000
# $20 = 0x00000065
# $21 = 0x0000AB1E
# $22 = 0x5CA10000
# $23 = 0x0000AB1E
# $24 = 0x5CA10000
# $25 = 0x5CA1AB1E

xor		$26, $25, $25    # $26 = 0
addu	$26, $16, $25    # + $16
addu	$26, $26, $17    # + $17
addu	$26, $18, $26    # + $18
addu	$26, $26, $19    # + $19
addu	$26, $20, $26    # + $20
addu	$26, $26, $21    # + $21
addu	$26, $22, $26	 # + $22
addu	$26, $26, $23	 # + $23
addu	$26, $24, $26	 # + $24
addu	$26, $26, $25	 # + $25
li		$8, 0x9517B6F8
addu	$26, $26, $8	# $26 = BA5EBA11
						# Now that you're done go outside and play some BA5EBA11 in the sunlight!

